searchNodes=[{"doc":"Provides an interface to query Airtable bases/tables, and an optional server to cache the results of a table into memory for faster access and to avoid Airtable API access limitations.The preferred mode of operation is to run in rate-limited and cached mode, to ensure that no API limitations are hit. The functions in the ExAirtable base module will operate through the rate-limiter and cache by default.If you wish to skip rate-limiting and caching, you can simply hit the Airtable API directly and get nicely-wrapped results by using the Table endpoints directly (see below for setup).In either the direct-to-API or cached-and-rate-limited case, this library provides an %Airtable.Record{} and %Airtable.List{} struct which mirror the API results that Airtable provides. All results will be wrapped in those structs. We (currently) make no effort to convert Airtable's generic data representation into an app's local data domain, but instead provide a sane default that can be easily extended (via Ecto schemas for example). We also provide some convenience methods for things like record field retrieval and relationship matching (see below).SetupGenerally, you'll need to do two things:Add a %ExAirtable.Config.Base{} into your application configuration for each Airtable base that you intend to reference.For each table within a base that you intend to query, define a module in your codebase that implements the ExAirtable.Table behaviour (by running use ExAirtable.Table and implementing the base() and table_name() callbacks).For example:defmodule MyApp.MyAirtable do use ExAirtable.Table def base, do: %ExAirtable.Config.Base{ id: &quot;your base ID&quot;, api_key: &quot;your api key&quot; } def name, do: &quot;Table Name&quot; endWith this module defined, you can hit the API directly (skipping rate-limiting and caching) like so:iex&gt; MyApp.MyAirtable.list() %Airtable.List{} iex&gt; MyApp.MyAirtable.retrieve(&quot;rec12345&quot;) %Airtable.Record{}Rate-Limiting and CachingActivating the ExAirtable.Supervisor in your supervision tree (passing it any tables you've defined as above) confers a few advantages:All tables are automatically synchronized to a local (ETS) cache for much faster local response times.All requests are automatically rate-limited (5 requests per second per Airtable base) so as not to exceed Airtable API rate limits.Record creation requests are automatically split into batches of 10 (Airtable will reject larger requests).To run a local caching server, you can include a reference to the ExAirtable supervisor in your supervision tree, for example:defmodule My.Application do use Application def start(_type, _args) do children = [ # ... # Configure caching and rate-limiting processes {ExAirtable.Supervisor, {[MyApp.MyAirtable, MyApp.MyOtherAirtable, ...], sync_rate: :timer.seconds(15)}}, # ... ] opts = [strategy: :one_for_one, name: MyApplication.Supervisor] Supervisor.start_link(children, opts) end # ... endNote that the :sync_rate (the rate at which tables are refreshed from Airtable) is optional and will default to 30 seconds if omitted. Once you have configured things this way, you can call ExAirtable directly, and get all of the speed and reliability benefits of caching and rate-limiting.iex&gt; ExAirtable.list(MyApp.MyAirtable) {:ok, %ExAirtable.Airtable.List{}} iex&gt; ExAirtable.retrieve(MyApp.MyAirtable, &quot;rec12345&quot;) {:ok, %ExAirtable.Airtable.Record{}}Examples + Playing AroundThe codebase includes an example Table (ExAirtable.Example.EnvTable) that you can use to play around and get an idea of how the system works. This module uses environment variables as configuration. The included Makefile provides some quick command-line tools to run tests and a console with those environment variables pre-loaded. Simply edit the relevant environment variables in Makefile to point to a valid base/table name, and you'll be able to interact directly like this:# first, run `make console`, then... # retrieve data directly from Airtable&#39;s API... iex&gt; EnvTable.list %ExAirtable.Airtable.List{records: [%Record{}, %Record{}, ...]} iex&gt; EnvTable.retrieve(&quot;rec12345&quot;) %ExAirtable.Airtable.Record{} # start a caching and rate-limiting server iex&gt; ExAirtable.Supervisor.start_link([EnvTable]) # get all records from the cache (without hitting the Airtable API) iex&gt; ExAirtable.list(EnvTable) %ExAirtable.Airtable.List{}Because certain tasks such as retrieving fields and finding related data happen so often, we put in a few convenience functions to make those jobs easier.# grab a field from a record iex&gt; ExAirtable.Airtable.Record.get(record, &quot;Users&quot;) [&quot;rec1234&quot;, &quot;rec3456&quot;] # find related table data based on a record ID iex&gt; ExAirtable.Airtable.List.filter_relations(list, &quot;Users&quot;, &quot;rec1234&quot;) [%Record{fields: %{&quot;Users&quot; =&gt; [&quot;rec1234&quot;, &quot;rec3456&quot;]}}, ...]See the ExAirtable.Airtable.List and ExAirtable.Airtable.Record module documentation for more information.","ref":"ExAirtable.html","title":"ExAirtable","type":"module"},{"doc":"Create one or more records in your Airtable from an %Airtable.List{} request. If your list includes more than 10 records, the request will be split so as not to be rejected by the Airtable API.This call is asynchronous, but the local cache will be automatically updated with any new records when the callback is successful.See Service.create/2 for more details.","ref":"ExAirtable.html#create/2","title":"ExAirtable.create/2","type":"function"},{"doc":"Delete a single record (by ID) from an Airtable. If successful, the record will be deleted from the cache as well.This call is asynchronous, but the local cache will be automatically updated when the callback is successful.","ref":"ExAirtable.html#delete/2","title":"ExAirtable.delete/2","type":"function"},{"doc":"Get all records from the given table module's cacheExamplesiex&gt; list(EnvTable) {:ok, %Airtable.List{}}","ref":"ExAirtable.html#list/1","title":"ExAirtable.list/1","type":"function"},{"doc":"Same as list/0, but raises on error.","ref":"ExAirtable.html#list!/1","title":"ExAirtable.list!/1","type":"function"},{"doc":"Retrieve a single record from the table module's cache.Examplesiex&gt; retrieve(EnvTable, &quot;recLIY1WLOs8ocOAq&quot;) {:ok, %Airtable.Record{}}","ref":"ExAirtable.html#retrieve/2","title":"ExAirtable.retrieve/2","type":"function"},{"doc":"Same as retrieve/1, but raises on error.","ref":"ExAirtable.html#retrieve!/2","title":"ExAirtable.retrieve!/2","type":"function"},{"doc":"Update a record in your Airtable. This call is asynchronous, but the local cache will be automatically updated when the callback is successful.One particular thing to note is that Airtable won't allow updates for records that pass calculated fields. The objectionable_fields: [&quot;list&quot;, &quot;of&quot;, &quot;fieldNames&quot;] option will allow you to point those out so that your update goes through.See Service.create/2 for more details about options that can be passed.","ref":"ExAirtable.html#update/3","title":"ExAirtable.update/3","type":"function"},{"doc":"Struct for an Airtable List of Records. This should directly match results being sent/returned from the Airtable REST API.","ref":"ExAirtable.Airtable.List.html","title":"ExAirtable.Airtable.List","type":"module"},{"doc":"Filter the records in a list by a given function. Returns an array of %ExAirtable.Airtable.Record{} structs.See Enum.filter for more examples.","ref":"ExAirtable.Airtable.List.html#filter_records/2","title":"ExAirtable.Airtable.List.filter_records/2","type":"function"},{"doc":"Find all records in a list that are related on a given field matching a given IDExamplesiex&gt; filter_relations(list, &quot;Users&quot;, &quot;rec1234&quot;) [%Record{fields: %{&quot;Users&quot; =&gt; [&quot;rec1234&quot;, &quot;rec456&quot;]}}, ...] iex&gt; filter_relations(list, &quot;Users&quot;, &quot;invalid_id&quot;) []","ref":"ExAirtable.Airtable.List.html#filter_relations/3","title":"ExAirtable.Airtable.List.filter_relations/3","type":"function"},{"doc":"Convert a typical Airtable JSON response into an %ExAirtable.Airtable.List{}. Any weird response will return an empty list.","ref":"ExAirtable.Airtable.List.html#from_map/1","title":"ExAirtable.Airtable.List.from_map/1","type":"function"},{"doc":"","ref":"ExAirtable.Airtable.List.html#t:t/0","title":"ExAirtable.Airtable.List.t/0","type":"type"},{"doc":"Struct for an Airtable Record. This should directly match the results returned from any Airtable REST API endpoint that returns records.","ref":"ExAirtable.Airtable.Record.html","title":"ExAirtable.Airtable.Record","type":"module"},{"doc":"Convert a typical Airtable JSON response into a %Record{}","ref":"ExAirtable.Airtable.Record.html#from_map/1","title":"ExAirtable.Airtable.Record.from_map/1","type":"function"},{"doc":"Retrieve a field from within a record's fieldsReturns nil by default if no field matches.","ref":"ExAirtable.Airtable.Record.html#get/3","title":"ExAirtable.Airtable.Record.get/3","type":"function"},{"doc":"","ref":"ExAirtable.Airtable.Record.html#t:t/0","title":"ExAirtable.Airtable.Record.t/0","type":"type"},{"doc":"Configuration struct for an Airtable &quot;base&quot;. This is extracted into a separate type/module so that it can be more easily centrlized in systems that implement multiple tables / bases.","ref":"ExAirtable.Config.Base.html","title":"ExAirtable.Config.Base","type":"module"},{"doc":"A configuration for an Airtable Base.","ref":"ExAirtable.Config.Base.html#t:t/0","title":"ExAirtable.Config.Base.t/0","type":"type"},{"doc":"Configuration struct for an Airtable &quot;base&quot;","ref":"ExAirtable.Config.Table.html","title":"ExAirtable.Config.Table","type":"module"},{"doc":"A configuration for an Airtable Table - a reference to an %ExAirtable.Config.Base{} and a name.","ref":"ExAirtable.Config.Table.html#t:t/0","title":"ExAirtable.Config.Table.t/0","type":"type"},{"doc":"This is useful for playing around in the console.It'll load up a table based on whatever you have in your config. Try make console to quickly load the defaults.","ref":"ExAirtable.Example.EnvTable.html","title":"ExAirtable.Example.EnvTable","type":"module"},{"doc":"Create a record in your Airtable. See Service.create/2 for details.","ref":"ExAirtable.Example.EnvTable.html#create/1","title":"ExAirtable.Example.EnvTable.create/1","type":"function"},{"doc":"Delete a single record (by ID) from an Airtable","ref":"ExAirtable.Example.EnvTable.html#delete/1","title":"ExAirtable.Example.EnvTable.delete/1","type":"function"},{"doc":"Get all records from your Airtable. See Service.list/3 for details.","ref":"ExAirtable.Example.EnvTable.html#list/1","title":"ExAirtable.Example.EnvTable.list/1","type":"function"},{"doc":"Similar to list/1, except results aren't automatically concatenated with multiple API requests. Typically called automatically by a TableSynchronizer","ref":"ExAirtable.Example.EnvTable.html#list_async/1","title":"ExAirtable.Example.EnvTable.list_async/1","type":"function"},{"doc":"Get a single record from your Airtable, matching by ID. See Service.retrieve/2 for details.","ref":"ExAirtable.Example.EnvTable.html#retrieve/1","title":"ExAirtable.Example.EnvTable.retrieve/1","type":"function"},{"doc":"Utility function to return the table struct","ref":"ExAirtable.Example.EnvTable.html#table/0","title":"ExAirtable.Example.EnvTable.table/0","type":"function"},{"doc":"Update a record in your Airtable. See Service.update for details.","ref":"ExAirtable.Example.EnvTable.html#update/2","title":"ExAirtable.Example.EnvTable.update/2","type":"function"},{"doc":"The RateLimiter ensures that no requests go to Airtable fast enough to trigger API rate-limiting.Requests will be handled immediately until more than 5 happen, at which point they go onto a queue which flushes every second. This ensures that no more than 5 requests per second will ever be sent to Airtable.There is one ExAirtable.RateLimiter.BaseQueue per base to track requests, since Airtable does rate-limiting per-base.","ref":"ExAirtable.RateLimiter.html","title":"ExAirtable.RateLimiter","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"ExAirtable.RateLimiter.html#child_spec/1","title":"ExAirtable.RateLimiter.child_spec/1","type":"function"},{"doc":"Add a request to the request buffer for a given table.Note that the request buffer is a MapSet - meaning that (exact) duplicate requests will be ignored.","ref":"ExAirtable.RateLimiter.html#request/2","title":"ExAirtable.RateLimiter.request/2","type":"function"},{"doc":"Pass a list of valid module names where the given modules have implemented the ExAirtable.Table behaviour.Internal state is a map of %BaseQueue{} structs where the key is the base ID.","ref":"ExAirtable.RateLimiter.html#start_link/1","title":"ExAirtable.RateLimiter.start_link/1","type":"function"},{"doc":"A Ratelimiter contains a map of %ExAirtable.RateLimiter.BaseQueues{}s.Each Airtable base has its own rate limit, which is why we operate at the base level for rate-limiting.","ref":"ExAirtable.RateLimiter.BaseQueue.html","title":"ExAirtable.RateLimiter.BaseQueue","type":"module"},{"doc":"Track demand from a single BaseQueue GenStage producer","ref":"ExAirtable.RateLimiter.BaseQueue.html#t:t/0","title":"ExAirtable.RateLimiter.BaseQueue.t/0","type":"type"},{"doc":"Defines a struct that is handy for passing data back and forth between functions, as data.&quot;Job&quot; = Module, Function, Arguments","ref":"ExAirtable.RateLimiter.Job.html","title":"ExAirtable.RateLimiter.Job","type":"module"},{"doc":"Run a given %Job{}.","ref":"ExAirtable.RateLimiter.Job.html#run/1","title":"ExAirtable.RateLimiter.Job.run/1","type":"function"},{"doc":"A single job to be run - a module, function, and argument.This is not to be confused with the mfa() type, which is a &quot;module, function, arity&quot; triplet. In this case we need to pass the actual arguments, and we'll let the system take care of pattern-matching for arity.","ref":"ExAirtable.RateLimiter.Job.html#t:t/0","title":"ExAirtable.RateLimiter.Job.t/0","type":"type"},{"doc":"The RateLimiter takes an %ExAirtable.RateLimiter.Request{}, runs its :job and (optionally) sends the results to the :callback function as arguments. Any arguments defined in the :callback %Job{} will be prepended to the function arguments, with the results of :job being the final argument passed.","ref":"ExAirtable.RateLimiter.Request.html","title":"ExAirtable.RateLimiter.Request","type":"module"},{"doc":"Create a request","ref":"ExAirtable.RateLimiter.Request.html#create/1","title":"ExAirtable.RateLimiter.Request.create/1","type":"function"},{"doc":"","ref":"ExAirtable.RateLimiter.Request.html#create/2","title":"ExAirtable.RateLimiter.Request.create/2","type":"function"},{"doc":"Run a given request, piping the result to the given callback function (if any).If arguments are given in the request.callback, then the result of running the request.job will be the final argument after the arguments specified in the callback.","ref":"ExAirtable.RateLimiter.Request.html#run/1","title":"ExAirtable.RateLimiter.Request.run/1","type":"function"},{"doc":"A request to an ExAirtable.RateLimiter.","ref":"ExAirtable.RateLimiter.Request.html#t:t/0","title":"ExAirtable.RateLimiter.Request.t/0","type":"type"},{"doc":"The Service defines methods to directly hit the Airtable API. Most methods take an Airtable.Config.Table{}, along with parameters to be forwarded to the REST API.These methods can be called directly, provided you have a valid %ExAirtable.Config.Table{} configuration. Alternatively, you can define a module that &quot;inherits&quot; this behavior - see ExAirtable.Table for more details.API results will be returned in ExAirtable.Airtable.List{} and ExAirtable.Airtable.Record{} structs, to match what the Airtable API returns.Examplesiex&gt; table = %ExAirtable.Config.Table{ base: %ExAirtable.Config.Base{ id: &quot;your base ID&quot;, api_key: &quot;your api key&quot; }, name: &quot;My Airtable Table Name&quot; } iex&gt; list(table) %Airtable.List{} iex&gt; retrieve(table, &quot;rec1234&quot;) %Airtable.Record{}","ref":"ExAirtable.Service.html","title":"ExAirtable.Service","type":"module"},{"doc":"Create a record in Airtable. Pass in a valid %ExAirtable.Airtable.List{} struct. Returns an %ExAirtable.Airtable.List{} on success.Exampleiex&gt; create(table, %List{}) %List{} iex&gt; create(table, list_with_errors) {:error, reason}","ref":"ExAirtable.Service.html#create/2","title":"ExAirtable.Service.create/2","type":"function"},{"doc":"Delete a single record (by ID) from an Airtable.Returns a custom map on success - see below.Exampleiex&gt; delete(table, &quot;recJmmAR0IzpaekBn&quot;) %{&quot;deleted&quot; =&gt; true, &quot;id&quot; =&gt; &quot;recJmmAR0IzpaekBn&quot;}","ref":"ExAirtable.Service.html#delete/2","title":"ExAirtable.Service.delete/2","type":"function"},{"doc":"Issues a DELETE request to the given url.Returns {:ok, response} if the request is successful, {:error, reason} otherwise.See request/5 for more detailed information.","ref":"ExAirtable.Service.html#delete/3","title":"ExAirtable.Service.delete/3","type":"function"},{"doc":"Issues a DELETE request to the given url, raising an exception in case of failure.If the request does not fail, the response is returned.See request!/5 for more detailed information.","ref":"ExAirtable.Service.html#delete!/3","title":"ExAirtable.Service.delete!/3","type":"function"},{"doc":"Issues a GET request to the given url.Returns {:ok, response} if the request is successful, {:error, reason} otherwise.See request/5 for more detailed information.","ref":"ExAirtable.Service.html#get/3","title":"ExAirtable.Service.get/3","type":"function"},{"doc":"Issues a GET request to the given url, raising an exception in case of failure.If the request does not fail, the response is returned.See request!/5 for more detailed information.","ref":"ExAirtable.Service.html#get!/3","title":"ExAirtable.Service.get!/3","type":"function"},{"doc":"Issues a HEAD request to the given url.Returns {:ok, response} if the request is successful, {:error, reason} otherwise.See request/5 for more detailed information.","ref":"ExAirtable.Service.html#head/3","title":"ExAirtable.Service.head/3","type":"function"},{"doc":"Issues a HEAD request to the given url, raising an exception in case of failure.If the request does not fail, the response is returned.See request!/5 for more detailed information.","ref":"ExAirtable.Service.html#head!/3","title":"ExAirtable.Service.head!/3","type":"function"},{"doc":"List all records in a table.Returns an %Airtable.List{} on success, and an {:error, reason} tuple on failure.Valid options are:params - any parameters you wish to send along to the Airtable API (for example view: &quot;My View&quot; or sort: &quot;My Field&quot;. See https://airtable.com/YOURBASEID/api/docs for details (in the &quot;List Records&quot; sections).Examplesiex&gt; list(table, params: %{view: &quot;My View Name&quot;}) %Airtable.List{}","ref":"ExAirtable.Service.html#list/2","title":"ExAirtable.Service.list/2","type":"function"},{"doc":"Similar to list/2, except results aren't automatically concatenated with multiple API requests. This is typically utilized by cache processes rather than called by hand - although it could be used by hand if you only want one page of results.","ref":"ExAirtable.Service.html#list_async/2","title":"ExAirtable.Service.list_async/2","type":"function"},{"doc":"Issues an OPTIONS request to the given url.Returns {:ok, response} if the request is successful, {:error, reason} otherwise.See request/5 for more detailed information.","ref":"ExAirtable.Service.html#options/3","title":"ExAirtable.Service.options/3","type":"function"},{"doc":"Issues a OPTIONS request to the given url, raising an exception in case of failure.If the request does not fail, the response is returned.See request!/5 for more detailed information.","ref":"ExAirtable.Service.html#options!/3","title":"ExAirtable.Service.options!/3","type":"function"},{"doc":"Issues a PATCH request to the given url.Returns {:ok, response} if the request is successful, {:error, reason} otherwise.See request/5 for more detailed information.","ref":"ExAirtable.Service.html#patch/4","title":"ExAirtable.Service.patch/4","type":"function"},{"doc":"Issues a PATCH request to the given url, raising an exception in case of failure.If the request does not fail, the response is returned.See request!/5 for more detailed information.","ref":"ExAirtable.Service.html#patch!/4","title":"ExAirtable.Service.patch!/4","type":"function"},{"doc":"Issues a POST request to the given url.Returns {:ok, response} if the request is successful, {:error, reason} otherwise.See request/5 for more detailed information.","ref":"ExAirtable.Service.html#post/4","title":"ExAirtable.Service.post/4","type":"function"},{"doc":"Issues a POST request to the given url, raising an exception in case of failure.If the request does not fail, the response is returned.See request!/5 for more detailed information.","ref":"ExAirtable.Service.html#post!/4","title":"ExAirtable.Service.post!/4","type":"function"},{"doc":"Callback implementation for HTTPoison.Base.process_headers/1.","ref":"ExAirtable.Service.html#process_headers/1","title":"ExAirtable.Service.process_headers/1","type":"function"},{"doc":"Callback implementation for HTTPoison.Base.process_request_body/1.","ref":"ExAirtable.Service.html#process_request_body/1","title":"ExAirtable.Service.process_request_body/1","type":"function"},{"doc":"Callback implementation for HTTPoison.Base.process_request_headers/1.","ref":"ExAirtable.Service.html#process_request_headers/1","title":"ExAirtable.Service.process_request_headers/1","type":"function"},{"doc":"Callback implementation for HTTPoison.Base.process_request_options/1.","ref":"ExAirtable.Service.html#process_request_options/1","title":"ExAirtable.Service.process_request_options/1","type":"function"},{"doc":"Callback implementation for HTTPoison.Base.process_request_params/1.","ref":"ExAirtable.Service.html#process_request_params/1","title":"ExAirtable.Service.process_request_params/1","type":"function"},{"doc":"Callback implementation for HTTPoison.Base.process_request_url/1.","ref":"ExAirtable.Service.html#process_request_url/1","title":"ExAirtable.Service.process_request_url/1","type":"function"},{"doc":"Callback implementation for HTTPoison.Base.process_response/1.","ref":"ExAirtable.Service.html#process_response/1","title":"ExAirtable.Service.process_response/1","type":"function"},{"doc":"Callback implementation for HTTPoison.Base.process_response_body/1.","ref":"ExAirtable.Service.html#process_response_body/1","title":"ExAirtable.Service.process_response_body/1","type":"function"},{"doc":"Callback implementation for HTTPoison.Base.process_response_chunk/1.","ref":"ExAirtable.Service.html#process_response_chunk/1","title":"ExAirtable.Service.process_response_chunk/1","type":"function"},{"doc":"Callback implementation for HTTPoison.Base.process_response_headers/1.","ref":"ExAirtable.Service.html#process_response_headers/1","title":"ExAirtable.Service.process_response_headers/1","type":"function"},{"doc":"Callback implementation for HTTPoison.Base.process_response_status_code/1.","ref":"ExAirtable.Service.html#process_response_status_code/1","title":"ExAirtable.Service.process_response_status_code/1","type":"function"},{"doc":"Callback implementation for HTTPoison.Base.process_status_code/1.","ref":"ExAirtable.Service.html#process_status_code/1","title":"ExAirtable.Service.process_status_code/1","type":"function"},{"doc":"Callback implementation for HTTPoison.Base.process_url/1.","ref":"ExAirtable.Service.html#process_url/1","title":"ExAirtable.Service.process_url/1","type":"function"},{"doc":"Issues a PUT request to the given url.Returns {:ok, response} if the request is successful, {:error, reason} otherwise.See request/5 for more detailed information.","ref":"ExAirtable.Service.html#put/4","title":"ExAirtable.Service.put/4","type":"function"},{"doc":"Issues a PUT request to the given url, raising an exception in case of failure.If the request does not fail, the response is returned.See request!/5 for more detailed information.","ref":"ExAirtable.Service.html#put!/4","title":"ExAirtable.Service.put!/4","type":"function"},{"doc":"Issues an HTTP request using a Request struct.This function returns {:ok, response} or {:ok, async_response} if the request is successful, {:error, reason} otherwise.Examplesrequest = %HTTPoison.Request{ method: :post, url: &quot;https://my.website.com&quot;, body: &quot;{\\&quot;foo\\&quot;: 3}&quot;, headers: [{&quot;Accept&quot;, &quot;application/json&quot;}] } request(request)","ref":"ExAirtable.Service.html#request/1","title":"ExAirtable.Service.request/1","type":"function"},{"doc":"Issues an HTTP request with the given method to the given url.This function is usually used indirectly by get/3, post/4, put/4, etcArgs:method - HTTP method as an atom (:get, :head, :post, :put, :delete, etc.)url - target url as a binary string or char listbody - request body. See more belowheaders - HTTP headers as an orddict (e.g., [{&quot;Accept&quot;, &quot;application/json&quot;}])options - Keyword list of optionsBody: see type HTTPoison.RequestOptions: see type HTTPoison.RequestThis function returns {:ok, response} or {:ok, async_response} if the request is successful, {:error, reason} otherwise.Examplesrequest(:post, &quot;https://my.website.com&quot;, &quot;{\\&quot;foo\\&quot;: 3}&quot;, [{&quot;Accept&quot;, &quot;application/json&quot;}])","ref":"ExAirtable.Service.html#request/5","title":"ExAirtable.Service.request/5","type":"function"},{"doc":"Issues an HTTP request with the given method to the given url, raising an exception in case of failure.request!/5 works exactly like request/5 but it returns just the response in case of a successful request, raising an exception in case the request fails.","ref":"ExAirtable.Service.html#request!/5","title":"ExAirtable.Service.request!/5","type":"function"},{"doc":"Retrieve a single record, matching by ID. Returns an %Airtable.Record{} on success and an {:error, reason} tuple on failure.","ref":"ExAirtable.Service.html#retrieve/2","title":"ExAirtable.Service.retrieve/2","type":"function"},{"doc":"Starts HTTPoison and its dependencies.","ref":"ExAirtable.Service.html#start/0","title":"ExAirtable.Service.start/0","type":"function"},{"doc":"Requests the next message to be streamed for a given HTTPoison.AsyncResponse.See request!/5 for more detailed information.","ref":"ExAirtable.Service.html#stream_next/1","title":"ExAirtable.Service.stream_next/1","type":"function"},{"doc":"Update a List{} of Record{}s in Airtable.Valid options include:objectionable_fields - A list of field names to remove from the fields entry of any Record in your List. This is typically used if you're trying to update in a table that doesn't allow updating certain (calculated) fields.overwrite - Will overwrite all values in the destination record with values being sent. Default is false, which will only update fields that have values (ie aren't null).","ref":"ExAirtable.Service.html#update/3","title":"ExAirtable.Service.update/3","type":"function"},{"doc":"","ref":"ExAirtable.Service.html#t:body/0","title":"ExAirtable.Service.body/0","type":"type"},{"doc":"","ref":"ExAirtable.Service.html#t:headers/0","title":"ExAirtable.Service.headers/0","type":"type"},{"doc":"","ref":"ExAirtable.Service.html#t:method/0","title":"ExAirtable.Service.method/0","type":"type"},{"doc":"","ref":"ExAirtable.Service.html#t:options/0","title":"ExAirtable.Service.options/0","type":"type"},{"doc":"","ref":"ExAirtable.Service.html#t:params/0","title":"ExAirtable.Service.params/0","type":"type"},{"doc":"","ref":"ExAirtable.Service.html#t:request/0","title":"ExAirtable.Service.request/0","type":"type"},{"doc":"","ref":"ExAirtable.Service.html#t:url/0","title":"ExAirtable.Service.url/0","type":"type"},{"doc":"This is the &quot;master control&quot; that takes care of rate-limiting and caching for all of your Tables. See the ExAirtable module and start_link/2 for details about initialization.","ref":"ExAirtable.Supervisor.html","title":"ExAirtable.Supervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"ExAirtable.Supervisor.html#child_spec/1","title":"ExAirtable.Supervisor.child_spec/1","type":"function"},{"doc":"Given a list of table module names, where each module has implemented the ExAirtable.Table behaviour...Start a global RateLimiter.For each passed table module, start a TableCache, which will in turn start a TableSynchronizer.Given options will be passed to sub-processes. Valid options are::sync_rate (integer) - amount of time in milliseconds between attempts to refresh table data from the API. Each table that you define will re-synchronize at this rate.","ref":"ExAirtable.Supervisor.html#start_link/1","title":"ExAirtable.Supervisor.start_link/1","type":"function"},{"doc":"The Table behaviour allows you to define your own modules that use Airtables. It is a thin wrapper around Service, but often more convenient to use.Examplesdefmodule MyTable do use ExAirtable.Table def base, do: %ExAirtable.Config.Base{ id: &quot;your base ID&quot;, api_key: &quot;your api key&quot; } def name, do: &quot;My Airtable Table Name&quot; end iex&gt; MyTable.list() %ExAirtable.Airtable.List{} iex&gt; MyTable.retrieve(&quot;rec123&quot;) %ExAirtable.Airtable.Record{}","ref":"ExAirtable.Table.html","title":"ExAirtable.Table","type":"behaviour"},{"doc":"A valid %ExAirtable.Config.Base{} config for your table","ref":"ExAirtable.Table.html#c:base/0","title":"ExAirtable.Table.base/0","type":"callback"},{"doc":"The name of your table within Airtable","ref":"ExAirtable.Table.html#c:name/0","title":"ExAirtable.Table.name/0","type":"callback"},{"doc":"A caching server for an ExAirtable.Table. Given a module name that implements the Table behaviour, it will automatically spawn synchronization processes and provide an in-memory data store that stays in sync with the external Airtable table/base.Examplesiex&gt; TableCache.retrieve(MyAirtableTable, &quot;rec1234&quot;) %Airtable.Record{} iex&gt; TableCache.list(MyAirtableTable) %Airtable.List{records: [%Airtable.Record{}, ...]}","ref":"ExAirtable.TableCache.html","title":"ExAirtable.TableCache","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"ExAirtable.TableCache.html#child_spec/1","title":"ExAirtable.TableCache.child_spec/1","type":"function"},{"doc":"Given an ExAirtable.Table module and an ID map, delete the record matching that ID. This is an asynchronous operation.","ref":"ExAirtable.TableCache.html#delete/2","title":"ExAirtable.TableCache.delete/2","type":"function"},{"doc":"Initialize the caching server.This is not meant to be called manually, but will be handle when start_link/1 is called.See start_link/1 for options.","ref":"ExAirtable.TableCache.html#init/1","title":"ExAirtable.TableCache.init/1","type":"function"},{"doc":"Given an ExAirtable.Table module, get all %Airtable.Record{}s in that cache's table as an %Airtable.List{}.","ref":"ExAirtable.TableCache.html#list/1","title":"ExAirtable.TableCache.list/1","type":"function"},{"doc":"Given an ExAirtable.Table module, and a list result, store that result for later cache replacement.Once we have the final page in a paginated list (or a list that has less than 100 records), we can replace the cache.This function is typically called by an async process to accumulate paginated data from Airtable's list method.","ref":"ExAirtable.TableCache.html#push_paginated_list/2","title":"ExAirtable.TableCache.push_paginated_list/2","type":"function"},{"doc":"Given an ExAirtable.Table module and a string key, get the %Record{} in that cache with the matching key.","ref":"ExAirtable.TableCache.html#retrieve/2","title":"ExAirtable.TableCache.retrieve/2","type":"function"},{"doc":"Given an ExAirtable.Table module, an ID, and a corresponding item to match that ID, store it in the cache.","ref":"ExAirtable.TableCache.html#set/3","title":"ExAirtable.TableCache.set/3","type":"function"},{"doc":"Replace an entire cache with a new set of %ExAirtable.Airtable.Record{}s.","ref":"ExAirtable.TableCache.html#set_all/2","title":"ExAirtable.TableCache.set_all/2","type":"function"},{"doc":"Start a caching server.Valid options (in the opts field) include:table_module (required) - The module name of the ExAirtable.Table module you're caching. If this is not included, the link will raise an error.sync_rate - How often (in ms) to refresh data from Airtable.skip_sync (boolean) - If you don't want to run the sync server for whatever reason (typically testing)","ref":"ExAirtable.TableCache.html#start_link/1","title":"ExAirtable.TableCache.start_link/1","type":"function"},{"doc":"Returns a valid ETS table name for a given ExAirtable.Table module.","ref":"ExAirtable.TableCache.html#table_for/1","title":"ExAirtable.TableCache.table_for/1","type":"function"},{"doc":"Given an ExAirtable.Table module and an ExAirtable.Airtable.Record{} struct, update the matching record in the cache, and insert it if it doesn't exist.","ref":"ExAirtable.TableCache.html#update/2","title":"ExAirtable.TableCache.update/2","type":"function"},{"doc":"A struct that contains the state for a TableCache.","ref":"ExAirtable.TableCache.html#t:t/0","title":"ExAirtable.TableCache.t/0","type":"type"},{"doc":"Run scheduled synchronization of an ExAirtable.TableCache against the relevant Airtable base. This will be automatically spawned and linked to an ExAirtable.TableCache when start_link/2 is run for that cache.","ref":"ExAirtable.TableSynchronizer.html","title":"ExAirtable.TableSynchronizer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"ExAirtable.TableSynchronizer.html#child_spec/1","title":"ExAirtable.TableSynchronizer.child_spec/1","type":"function"},{"doc":"","ref":"ExAirtable.TableSynchronizer.html#start_link/1","title":"ExAirtable.TableSynchronizer.start_link/1","type":"function"},{"doc":"A struct that contains the state for a TableSynchronizer","ref":"ExAirtable.TableSynchronizer.html#t:t/0","title":"ExAirtable.TableSynchronizer.t/0","type":"type"}]